<?php
/**
 * XmonApi
 */
class XmonApi
{
    const JSONRPC = '2.0';
    const TYPE = 'sdk';
    
    /**
     * @var API URL.
     */
    private $url;
    
    /**
     * @var json request user_id
     */
    private $userId;
    
    /**
     * @var json request auth_token
     */
    private $authToken;
    
    /**
     * @var json request method
     */
    private $method;
    
    /**
     * @var json request params
     */
    private $params;
    
    /**
     * set method
     * 
     * @param string $method Request method name.
     * 
     * @return void
     */
    public function setMethod($method = '')
    {
        $this->method = $method;
    }
    
    /**
     * set params
     * 
     * @param string $method Request params.
     * 
     * @return void
     */
    public function setParams($params = array())
    {
        $this->params = $params;
    }
    
    /**
     * request
     * 
     * @throws Exception
     * 
     * @return array json response data.
     */
    public function request()
    {
        if (!strlen($this->method)) {
            throw new Exception('Not specified a method.' . "\n");
        }
        
        $id = sha1(time() . mt_rand());
        
        $content = array(
            'jsonrpc' => XmonApi::JSONRPC,
            'method' => $this->method,
            'params' => $this->params,
            'id' => $id,
            'user_id' => $this->userId,
            'auth_token' => $this->authToken,
            'type' => XmonApi::TYPE
        );
        $encodeContents = json_encode($content);
        
        $opts = array(
            'http' => array(
                'method' => 'POST',
                'header' => "Content-type: application/json-rpc\r\n",
                'content' => $encodeContents
            )
        );
        
        $context = stream_context_create($opts);
        $fp = fopen($this->url, 'r', false, $context);
        if (!$fp) {
            throw new Exception('Could not connect to ' . $this->url . "\n");
        }
        $result = @stream_get_contents($fp);
        
        if ($result === false) {
            throw new Exception('Could not connect to ' . $this->url . "\n");
        }
        $result = json_decode($result, true);
        if (is_null($result)) throw new Exception('Not found result data.' . "\n");
        
        return $result;
     }
     
     /**
      * constructor
      * 
      * @param string $url       X-MON URL.
      * @param string $userId    json request user_id.
      * @param string $authToken json request auth_token.
      * 
      * @return void
      */
     public function __construct($url, $userId, $authToken)
     {
        $this->url = $url;
        $this->userId = $userId;
        $this->authToken = $authToken;
        $this->setMethod();
        $this->setParams();
     }
}
